@@ -119,6 +119,9 @@ typedef enum {
 	GDM_ID_PAM_STACK,
 	GDM_ID_NEVER_PLACE_COOKIES_ON_NFS,
 	GDM_ID_PASSWORD_REQUIRED,
+	GDM_ID_UTMP_LINE_ATTACHED,
+	GDM_ID_UTMP_LINE_REMOTE,
+	GDM_ID_UTMP_PSEUDO_DEVICE,
 	GDM_ID_XDMCP,
 	GDM_ID_MAX_PENDING,
 	GDM_ID_MAX_SESSIONS,
@@ -366,6 +369,9 @@ static const GdmConfigEntry gdm_daemon_config_entries [] = {
 
 	{ GDM_CONFIG_GROUP_SECURITY, "NeverPlaceCookiesOnNFS", GDM_CONFIG_VALUE_BOOL, "true", GDM_ID_NEVER_PLACE_COOKIES_ON_NFS },
 	{ GDM_CONFIG_GROUP_SECURITY, "PasswordRequired", GDM_CONFIG_VALUE_BOOL, "false", GDM_ID_PASSWORD_REQUIRED },
+	{ GDM_CONFIG_GROUP_SECURITY, "UtmpLineAttached", GDM_CONFIG_VALUE_STRING, "", GDM_ID_UTMP_LINE_ATTACHED },
+	{ GDM_CONFIG_GROUP_SECURITY, "UtmpLineRemote", GDM_CONFIG_VALUE_STRING, "", GDM_ID_UTMP_LINE_REMOTE },
+	{ GDM_CONFIG_GROUP_SECURITY, "UtmpPseudoDevice", GDM_CONFIG_VALUE_BOOL, "", GDM_ID_UTMP_PSEUDO_DEVICE },
 
 	{ GDM_CONFIG_GROUP_XDMCP, "Enable", GDM_CONFIG_VALUE_BOOL, "false", GDM_ID_XDMCP },
 	{ GDM_CONFIG_GROUP_XDMCP, "MaxPending", GDM_CONFIG_VALUE_INT, "4", GDM_ID_MAX_PENDING },
