@@ -19,22 +19,32 @@
 #include "config.h"
 
 #include <unistd.h>
-#include <syslog.h>
+#include <string.h>
+
+#ifdef HAVE_CHKAUTHATTR
+#include <auth_attr.h>
+#include <secdb.h>
+#endif
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
+#include "gdm.h"
+#include "gdmcommon.h"
+#include "gdmconfig.h"
+#include "gdmwm.h"
+#include "misc.h"
+
+#include "gdm-common.h"
+#include "gdm-socket-protocol.h"
+#include "gdm-daemon-config-keys.h"
+
 #include "greeter.h"
 #include "greeter_configuration.h"
 #include "greeter_system.h"
 #include "greeter_item.h"
 #include "greeter_item_ulist.h"
 #include "greeter_parser.h"
-#include "gdm.h"
-#include "gdmcommon.h"
-#include "gdmconfig.h"
-#include "gdmwm.h"
-#include "misc.h"
 
 GtkWidget       *dialog;
 extern gboolean  GdmHaltFound;
@@ -74,8 +84,6 @@ query_greeter_restart_handler (void)
 {
 	if (gdm_wm_warn_dialog (_("Are you sure you want to restart the computer?"), "",
 			     _("_Restart"), NULL, TRUE) == GTK_RESPONSE_YES) {
-		closelog ();
-		
 		_exit (DISPLAY_REBOOT);
 	}
 }
@@ -85,7 +93,7 @@ query_greeter_custom_cmd_handler (GtkWidget *widget, gpointer data)
 {
         if (data) {
 		gint *cmd_id = (gint*)data;
-	        gchar * key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, *cmd_id);
+	        gchar * key_string = g_strdup_printf ("%s%d=", GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, *cmd_id);
 		if (gdm_wm_warn_dialog (gdm_config_get_string (key_string) , "", 
 					GTK_STOCK_OK, NULL, TRUE) == GTK_RESPONSE_YES) {
 		        printf ("%c%c%c%d\n", STX, BEL, GDM_INTERRUPT_CUSTOM_CMD, *cmd_id);
@@ -100,8 +108,6 @@ query_greeter_halt_handler (void)
 {
 	if (gdm_wm_warn_dialog (_("Are you sure you want to Shut Down the computer?"), "",
 			     _("Shut _Down"), NULL, TRUE) == GTK_RESPONSE_YES) {
-		closelog ();
-
 		_exit (DISPLAY_HALT);
 	}
 }
@@ -120,7 +126,6 @@ query_greeter_suspend_handler (void)
 static void
 greeter_restart_handler (void)
 {
-	closelog ();
 	_exit (DISPLAY_REBOOT);
 }
 
@@ -134,7 +139,6 @@ greeter_custom_cmd_handler (gint cmd_id)
 static void
 greeter_halt_handler (void)
 {
-	closelog ();
 	_exit (DISPLAY_HALT);
 }
 
@@ -164,10 +168,66 @@ greeter_config_handler (void)
 static void
 greeter_chooser_handler (void)
 {
-	closelog ();
 	_exit (DISPLAY_RUN_CHOOSER);
 }
 
+static gboolean
+is_action_available (gchar *action)
+{
+	gchar **allowsyscmd = NULL;
+	const gchar *allowsyscmdval;
+	gboolean ret = FALSE;
+	int i;
+
+	allowsyscmdval = gdm_config_get_string (GDM_KEY_SYSTEM_COMMANDS_IN_MENU);
+	if (allowsyscmdval)
+		allowsyscmd = g_strsplit (allowsyscmdval, ";", 0);
+
+	if (allowsyscmd) {
+		for (i = 0; allowsyscmd[i] != NULL; i++) {
+			if (strcmp (allowsyscmd[i], action) == 0) {
+				ret = TRUE;
+				break;
+			}
+		}
+	}
+
+#ifdef HAVE_CHKAUTHATTR
+	if (ret == TRUE) {
+		gchar **rbackeys = NULL;
+		const gchar *rbackeysval;
+		const char *gdmuser;
+
+		gdmuser     = gdm_config_get_string (GDM_KEY_USER);
+		rbackeysval = gdm_config_get_string (GDM_KEY_RBAC_SYSTEM_COMMAND_KEYS);
+		if (rbackeysval)
+			rbackeys = g_strsplit (rbackeysval, ";", 0);
+
+		if (rbackeys) {
+			for (i = 0; rbackeys[i] != NULL; i++) {
+				gchar **rbackey = g_strsplit (rbackeys[i], ":", 2);
+
+				if (! ve_string_empty (rbackey[0]) &&
+				    ! ve_string_empty (rbackey[1]) &&
+				    strcmp (rbackey[0], action) == 0) {
+
+					if (!chkauthattr (rbackey[1], gdmuser)) {
+						g_strfreev (rbackey);
+						ret = FALSE;
+						break;
+					}
+				}
+				g_strfreev (rbackey);
+			}
+		}
+		g_strfreev (rbackeys);
+	}
+#endif
+	g_strfreev (allowsyscmd);
+
+	return ret;
+}
+
 void
 greeter_system_append_system_menu (GtkWidget *menu)
 {
@@ -180,7 +240,10 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		return;
 
 	if (gdm_config_get_bool (GDM_KEY_CHOOSER_BUTTON)) {
-		w = gtk_menu_item_new_with_mnemonic (_("Remote Login via _XDMCP..."));
+		w = gtk_image_menu_item_new_with_mnemonic (_("Remote Login via _XDMCP..."));
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w),
+			gtk_image_new_from_icon_name ("preferences-desktop-remote-desktop", GTK_ICON_SIZE_MENU));
+
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
 		g_signal_connect (G_OBJECT (w), "activate",
@@ -188,9 +251,17 @@ greeter_system_append_system_menu (GtkWidget *menu)
 				  NULL);
 	}
 
+	/*
+	 * Disable Configuration if using accessibility (AddGtkModules) since
+	 * using it with accessibility causes a hang.
+	 */
 	if (gdm_config_get_bool (GDM_KEY_CONFIG_AVAILABLE) &&
+	    !gdm_config_get_bool (GDM_KEY_ADD_GTK_MODULES) &&
 	    bin_exists (gdm_config_get_string (GDM_KEY_CONFIGURATOR))) {
-		w = gtk_menu_item_new_with_mnemonic (_("Confi_gure Login Manager..."));
+		w = gtk_image_menu_item_new_with_mnemonic (_("Confi_gure Login Manager..."));
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w),
+			gtk_image_new_from_icon_name ("gdm-setup", GTK_ICON_SIZE_MENU));
+
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
 		g_signal_connect (G_OBJECT (w), "activate",
@@ -204,8 +275,10 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		gtk_widget_show (sep);
 	}
 
-	if (GdmRebootFound) {
-		w = gtk_menu_item_new_with_mnemonic (_("_Restart"));
+	if (GdmRebootFound && is_action_available ("REBOOT")) {
+ 		w = gtk_image_menu_item_new_with_mnemonic (_("_Restart"));
+ 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w),
+ 					       gtk_image_new_from_icon_name ("system-restart", GTK_ICON_SIZE_MENU));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
 		g_signal_connect (G_OBJECT (w), "activate",
@@ -213,13 +286,35 @@ greeter_system_append_system_menu (GtkWidget *menu)
 				  NULL);
 	}
 
-	if (GdmAnyCustomCmdsFound) {
+	if (GdmHaltFound && is_action_available ("HALT")) {
+ 		w = gtk_image_menu_item_new_with_mnemonic (_("Shut _Down"));
+ 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w), 
+ 					       gtk_image_new_from_icon_name ("system-shut-down", GTK_ICON_SIZE_MENU));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
+		gtk_widget_show (GTK_WIDGET (w));
+		g_signal_connect (G_OBJECT (w), "activate",
+				  G_CALLBACK (query_greeter_halt_handler),
+				  NULL);
+	}
+
+	if (GdmSuspendFound && is_action_available ("SUSPEND")) {
+ 		w = gtk_image_menu_item_new_with_mnemonic (_("Sus_pend"));
+ 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w),
+ 					       gtk_image_new_from_icon_name ("system-suspend", GTK_ICON_SIZE_MENU));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
+		gtk_widget_show (GTK_WIDGET (w));
+		g_signal_connect (G_OBJECT (w), "activate",
+				  G_CALLBACK (query_greeter_suspend_handler),
+				  NULL);
+	}
+
+	if (GdmAnyCustomCmdsFound && is_action_available ("CUSTOM_CMD")) {
 		for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
 		        if (GdmCustomCmdsFound[i]){
 				gint * cmd_index = g_new0(gint, 1);
-				*cmd_index = i;
 				gchar * key_string = NULL;
-				key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
+				*cmd_index = i;
+				key_string = g_strdup_printf ("%s%d=", GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
 				w = gtk_menu_item_new_with_mnemonic (gdm_config_get_string(key_string));
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 				gtk_widget_show (GTK_WIDGET (w));
@@ -231,23 +326,6 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		}
 	}
 
-	if (GdmHaltFound) {
-		w = gtk_menu_item_new_with_mnemonic (_("Shut _Down"));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
-		gtk_widget_show (GTK_WIDGET (w));
-		g_signal_connect (G_OBJECT (w), "activate",
-				  G_CALLBACK (query_greeter_halt_handler),
-				  NULL);
-	}
-
-	if (GdmSuspendFound) {
-		w = gtk_menu_item_new_with_mnemonic (_("Sus_pend"));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
-		gtk_widget_show (GTK_WIDGET (w));
-		g_signal_connect (G_OBJECT (w), "activate",
-				  G_CALLBACK (query_greeter_suspend_handler),
-				  NULL);
-	}
 }
 
 static gboolean
@@ -366,13 +444,13 @@ greeter_system_handler (GreeterItemInfo *info,
 	          custom_cmds_radio[i] = NULL;
 		  if (GdmCustomCmdsFound[i]){
 		          gchar * key_string = NULL;
-			  key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
+			  key_string = g_strdup_printf ("%s%d=", GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
 			  radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (group_radio));
 			  custom_cmds_radio[i] = gtk_radio_button_new_with_mnemonic (radio_group,
 										     gdm_config_get_string(key_string));
 			  group_radio = custom_cmds_radio[i];
 			  g_free (key_string);
-			  key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
+			  key_string = g_strdup_printf ("%s%d=", GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
 			  gtk_tooltips_set_tip (tooltips, GTK_WIDGET (custom_cmds_radio[i]),
 						gdm_config_get_string(key_string),
 						NULL);
@@ -423,7 +501,12 @@ greeter_system_handler (GreeterItemInfo *info,
 	  gtk_widget_show (chooser_radio);
   }
 
+  /*
+   * Disable Configuration if using accessibility (AddGtkModules) since
+   * using it with accessibility causes a hang.
+   */
   if (gdm_config_get_bool (GDM_KEY_CONFIG_AVAILABLE) &&
+      !gdm_config_get_bool (GDM_KEY_ADD_GTK_MODULES) &&
       bin_exists (gdm_config_get_string (GDM_KEY_CONFIGURATOR))) {
 	  if (group_radio != NULL)
 		  radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (group_radio));
@@ -512,8 +595,9 @@ greeter_item_system_setup (void)
 
   for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
 	  gint * cmd_index = g_new0(gint, 1);
+	  gchar * key_string;
 	  *cmd_index = i;
-	  gchar * key_string = g_strdup_printf (_("custom_cmd_button%d"), i);
+	  key_string = g_strdup_printf ("custom_cmd_button%d", i);
 	  greeter_item_register_action_callback (key_string,
 						 (ActionFunc)query_greeter_custom_cmd_handler,
 						 cmd_index);
